#include <Arduino.h>

#define RF_TX_PORT 7

struct Protocol {
  String Name;
  int Delay_0;  //Delay when 0 (in uSeconds)
  int Delay_1;  //Delay when 1 (in uSeconds)
  int Repeats;  //Number of times to repeat the transmission
  String CommonStream;
  String OnStream;
  String OffStream;
};

struct Unit {
  int SensorNb;
  String AddressStream;
  Protocol *protocol;
};

enum UnitState {ON, OFF};

//Declare protocol for different units types
Protocol Anslut = 
{
   "Anslut", 
   310,        //Pulswidth on 0
   310,        //Pulswidth on 1
   6,          //Number of repeats 
   "0111111111010111101111010111101010111101111010111101010111101011110111101011110101111010111101010111101111010101111010111101111010111101011110101111010111101011110101011110111101011110101011110101111"
   "oooooooo11110101111010aaaaaaaaaaaaaa",
   "01111010", //Code for set switch ON
   "01011110"  //Code for set switch OFF
};

Protocol OldBig = 
{
   "OldBig", 
   145,
   115,
   15,
   "01001001001001aaaa0100100100100100ooo",
   "110",
   "100"
};


class My433
{
   public:
      My433()
      {
         pinMode(RF_TX_PORT, OUTPUT);
      }

      void SetUnitON(Unit *unit)
      {
         sendSequence(unit, ON);
      }

      void SetUnitOFF(Unit *unit)
      {
         sendSequence(unit, OFF);
      }

   private:
      void sendSequence(Unit *unitToSend, UnitState state)
      {
         for (int i=0; i<unitToSend->protocol->Repeats; i++)
         {
            sendRF(unitToSend, state);
            delay(5); //5 ms delay between bursts
         }
      }

      void sendRF(Unit *unitToSend, UnitState state)
      {
         String sendStream = "";
         digitalWrite(RF_TX_PORT, LOW);
         int onOffIndex = 0;
         int addressIndex = 0;
         for (int c=0; c < unitToSend->protocol->CommonStream.length(); c++)
         {
            char data = unitToSend->protocol->CommonStream[c];
            //if (data == 'x')
            //{
            //   data = state==ON ? unitToSend->UnitStreamON[unitStreamIndex] : unitToSend->UnitStreamOFF[unitStreamIndex];
            //   unitStreamIndex++;
            //}
            if (data == 'o')
            {
               data = state==ON ? unitToSend->protocol->OnStream[onOffIndex] : unitToSend->protocol->OffStream[onOffIndex];
               onOffIndex++;
            }
            else if (data == 'a')
            {
               data = unitToSend->AddressStream[addressIndex];
               addressIndex++;
            }

            if (data == '0')
            {
               digitalWrite(RF_TX_PORT, HIGH);
               delayMicroseconds(unitToSend->protocol->Delay_0);
            }
            else
            {
               digitalWrite(RF_TX_PORT, LOW);
               delayMicroseconds(unitToSend->protocol->Delay_1);
            }
            sendStream += data;
         }
         digitalWrite(RF_TX_PORT, LOW);
         Serial.println("Send: " + sendStream);
      }
};
